@@ -3,7 +3,7 @@ import collections
 import typing as tp
 import warnings
 
-from ..recast_exceptions import rethrow_as
+from ..recast_exceptions import rethrow_as, silence_excs
 from ..decorators import for_argument
 
 T, U = tp.TypeVar('T'), tp.TypeVar('U')
@@ -138,25 +138,26 @@ def even(sq: IteratorOrIterable) -> tp.Iterator[T]:
     Return only elements with even indices in this iterable (first element will be returned,
     as indices are counted from 0)
     """
-    try:
-        while True:
+    while True:
+        try:
             yield next(sq)
             next(sq)
-    except StopIteration:
-        return
+        except StopIteration:
+            return
 
 
 @for_argument(iter)
+@silence_excs(StopIteration)
 def odd(sq: IteratorOrIterable) -> tp.Iterator[T]:
     """
     Return only elements with odd indices in this iterable.
     """
-    try:
-        while True:
+    while True:
+        try:
             next(sq)
             yield next(sq)
-    except StopIteration:
-        return
+        except StopIteration:
+            return
 
 
 def count(sq: IteratorOrIterable, start: tp.Optional[int] = None, step: int = 1,
@@ -216,6 +217,7 @@ def is_instance(classes: tp.Union[tp.Tuple[type, ...], type]) -> tp.Callable[[ob
     return inner
 
 
+@for_argument(iter, iter)
 def other_sequence_no_longer_than(base_sequence: IteratorOrIterable,
                                   other_sequence: IteratorOrIterable) -> tp.Iterator[T]:
     """
@@ -226,7 +228,6 @@ def other_sequence_no_longer_than(base_sequence: IteratorOrIterable,
     :param base_sequence: sequence whose p_len should be taken
     :param other_sequence: sequence to output values from
     """
-    base_sequence, other_sequence = iter(base_sequence), iter(other_sequence)
     while True:
         try:
             next(base_sequence)
@@ -270,6 +271,7 @@ def shift(iterable_: IteratorOrIterable, shift_factor: int) -> tp.Iterator[T]:
             return iterator
 
 
+@silence_excs(StopIteration)
 def zip_shifted(*args: tp.Union[IteratorOrIterable, tp.Tuple[IteratorOrIterable, int]]) -> \
         tp.Iterator[tp.Tuple[T, ...]]:
     """
@@ -292,7 +294,8 @@ def zip_shifted(*args: tp.Union[IteratorOrIterable, tp.Tuple[IteratorOrIterable,
     :param args: a tuple with the iterator/iterable and amount of shift. If a non-tuple is given,
         it is assumed that the shift is zero.
     """
-    warnings.warn('Use zip(shift(...)) instead!', DeprecationWarning)
+    warnings.warn('This is deprecated and will be removed in Satella 3.0. '
+                  'Use zip(shift(...)) instead!', DeprecationWarning)
 
     iterators = []  # type: tp.List[tp.Union[tp.Tuple[tp.Iterator[T], tp.List[T]], tp.Iterator[T]]
     for row in args:
@@ -304,28 +307,41 @@ def zip_shifted(*args: tp.Union[IteratorOrIterable, tp.Tuple[IteratorOrIterable,
     return zip(*iterators)
 
 
+@for_argument(iter)
+@silence_excs(StopIteration)
 def skip_first(iterator: IteratorOrIterable, n: int) -> tp.Iterator[T]:
     """
-    Skip first n elements from given iterator
+    Skip first n elements from given iterator.
+
+    Returned iterator may be empty, if source iterator is shorter or equal to n.
     """
-    iterator = iter(iterator)
+    warnings.warn('This is deprecated and will be removed in Satella 3.0. '
+                  'Please use itertools.islice instead', DeprecationWarning)
+
     for i in range(n):
         next(iterator)
     yield from iterator
 
 
+@for_argument(iter)
+@silence_excs(StopIteration)
 def stop_after(iterator: IteratorOrIterable, n: int) -> tp.Iterator[T]:
     """
     Stop this iterator after returning n elements, even if it's longer than that.
 
+    The resulting iterator may be shorter than n, if the source element is so.
+
     :param iterator: iterator or iterable to examine
     :param n: elements to return
     """
-    iterator = iter(iterator)
+    warnings.warn('This is deprecated and will be removed in Satella 3.0. '
+                  'Please use itertools.islice instead', DeprecationWarning)
+
     for i in range(n):
         yield next(iterator)
 
 
+@for_argument(iter)
 def n_th(iterator: IteratorOrIterable, n: int = 0) -> T:
     """
     Obtain n-th element (counting from 0) of an iterable
@@ -334,16 +350,53 @@ def n_th(iterator: IteratorOrIterable, n: int = 0) -> T:
     :param n: element to return. Note that we're counting from 0
     :raises IndexError: iterable was too short
     """
-    obj = iter(iterator)
     try:
         for i in range(n):
-            next(obj)
-        return next(obj)
+            next(iterator)
+        return next(iterator)
     except (StopIteration, GeneratorExit):
         raise IndexError('Iterable was too short')
 
 
-def satella_enumerate(iterator: IteratorOrIterable, start: int = 0) -> tp.Iterator[tp.Tuple]:
+def map_list(fun: tp.Callable, iterable: IteratorOrIterable) -> tp.List:
+    """
+    A syntactic sugar for
+
+    >>> list(map(fun, iterable))
+
+    :param fun: function to apply
+    :param iterable: iterable to iterate over
+    """
+    return list(map(fun, iterable))
+
+
+def smart_zip(*iterators: IteratorOrIterable) -> tp.Iterator[tp.Tuple[T, ...]]:
+    """
+    Zip in such a way that resulted tuples are automatically expanded.
+
+    Ie:
+
+    >>> b = list(smart_zip([(1, 1), (1, 2)], [1, 2]))
+    >>> assert b == [(1, 1, 1), (1, 2, 2)]
+
+    Note that an element of the zipped iterator must be a tuple (ie. isinstance tuple)
+    in order for it to be appended to resulting iterator element!
+
+    :param iterators: list of iterators to zip together
+    :return: an iterator zipping the arguments in a smart way
+    """
+    for row in zip(*iterators):
+        a = []
+        for elem in row:
+            if isinstance(elem, tuple):
+                a.extend(elem)
+            else:
+                a.append(elem)
+        yield tuple(a)
+
+
+def smart_enumerate(iterator: IteratorOrIterable, start: int = 0,
+                    step: int = 1) -> tp.Iterator[tp.Tuple]:
     """
     An enumerate that talks pretty with lists of tuples. Consider
 
@@ -359,6 +412,9 @@ def satella_enumerate(iterator: IteratorOrIterable, start: int = 0) -> tp.Iterat
     Note that elements in your iterable must be either a list of a tuple for that to work,
     or need to be able to be coerced to a tuple. Otherwise, TypeError will be thrown.
 
+    :param iterator: iterator to enumerate
+    :param start: value to start counting at
+    :param step: step to advance the enumeration with
     :raise TypeError: could not coerce the elements in your iterable to a tuple
     """
     i = start
@@ -367,11 +423,10 @@ def satella_enumerate(iterator: IteratorOrIterable, start: int = 0) -> tp.Iterat
             yield (i,) + row
         else:
             yield (i,) + tuple(row)
-        i += 1
-
-satella_enumerate.__name__ = 'enumerate'
+        i += step
 
 
+@for_argument(iter)
 def take_n(iterator: IteratorOrIterable, n: int, skip: int = 0) -> tp.List[T]:
     """
     Take (first) n elements of an iterator, or the entire iterator, whichever comes first
@@ -381,7 +436,6 @@ def take_n(iterator: IteratorOrIterable, n: int, skip: int = 0) -> tp.List[T]:
     :param skip: elements from the start to skip
     :return: list of p_len n (or shorter)
     """
-    iterator = iter(iterator)
     for i in range(skip):
         next(iterator)
 
